﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace PotterIMGView
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();
		}

		byte[] imgdata;
		byte[] pixdata;

		private void Form1_Load(object sender, EventArgs e)
		{
			if (Program.Arguments.Count > 0)
				LoadImage(Program.Arguments[0]);
		}

		private void LoadImage(string filename)
		{
			imgdata = File.ReadAllBytes(filename);
			ImageChanged();
		}

		private void button1_Click(object sender, EventArgs e)
		{
			using (OpenFileDialog dlg = new OpenFileDialog() { DefaultExt = "img", Filter = "IMG Files|*.img" })
				if (dlg.ShowDialog(this) == DialogResult.OK)
					LoadImage(dlg.FileName);
		}

		private void ImageChanged()
		{
			if (imgdata == null)
				return;
			using (MemoryStream str = new MemoryStream(imgdata))
			using (BinaryReader br = new BinaryReader(str))
			{
				if (checkBox1.Checked)
				{
					ushort[] pal = new ushort[256];
					for (int i = 0; i < 256; i++)
						pal[i] = BGR555ToRGB555(br.ReadUInt16());
					var imglength = str.Length - 512;
					pixdata = new byte[imglength * 2];
					for (int i = 0; i < imglength; i++)
						BitConverter.GetBytes(pal[br.ReadByte()]).CopyTo(pixdata, i * 2);
				}
				else
				{
					pixdata = new byte[str.Length];
					for (int i = 0; i < str.Length / 2; i++)
						BitConverter.GetBytes(BGR555ToRGB555(br.ReadUInt16())).CopyTo(pixdata, i * 2);
				}
			}
			DrawImage();
		}

		private static ushort BGR555ToRGB555(ushort s)
		{
			return (ushort)((s >> 10) | (s & 0x3E0) | ((s & 0x1F) << 10));
		}

		private void checkBox1_CheckedChanged(object sender, EventArgs e)
		{
			ImageChanged();
		}

		private void DrawImage()
		{
			if (pixdata == null)
				return;
			int width = (int)numericUpDown1.Value;
			int height = (int)Math.Ceiling((pixdata.Length / 2) / (double)width);
			Bitmap bmp = new Bitmap(width, height, PixelFormat.Format16bppRgb555);
			BitmapData bmpd = bmp.LockBits(new Rectangle(0, 0, width, height), ImageLockMode.WriteOnly, PixelFormat.Format16bppRgb555);
			Marshal.Copy(pixdata, 0, bmpd.Scan0, pixdata.Length);
			bmp.UnlockBits(bmpd);
			pictureBox1.Image = bmp;
		}

		private void numericUpDown1_ValueChanged(object sender, EventArgs e)
		{
			DrawImage();
		}

		private void pictureBox1_DoubleClick(object sender, EventArgs e)
		{
			using (SaveFileDialog dlg = new SaveFileDialog() { DefaultExt = "png", Filter = "PNG Files|*.png" })
				if (dlg.ShowDialog(this) == DialogResult.OK)
					pictureBox1.Image.Save(dlg.FileName);
		}
	}
}